import { Declare, SingleData, Widget } from 'ptnl-constructor-sdk';

import { EBlockKey, EViewOption } from './enum';
import * as echarts from 'echarts/core';
import { Text } from 'ptnl-constructor-sdk/config';;
import { yDigitsGroup } from './view-settings';

@Declare({
    provideCssVariables: true,
})
export class IndicatorGrad extends Widget implements SingleData {
    readonly data!: SingleData['data'];
    readonly dataSettings!: SingleData['dataSettings'];

    private check_is_all_null(row:any,keys:any){
        for(let k of keys){
            if(!(row[k]==undefined)){
                return false;
            }
        }
        return true;
    }

    private buildData(){
        let ret:any[]=[];
        let value_col=this.dataSettings.columnsByBlock[EBlockKey.Value][0].path;
        for(let row of this.data){
            let c_lvl:{children:any[],value?:number}={
                children:ret,
            }
            for(let c_num=0;c_num<this.dataSettings.columnsByBlock[EBlockKey.Series].length;c_num++ ){
                let col=this.dataSettings.columnsByBlock[EBlockKey.Series][c_num];
                if(row[col.path]==undefined){
                    if(c_num==this.dataSettings.columnsByBlock[EBlockKey.Series].length-1){
                        break;
                    }
                    if(this.check_is_all_null(row,this.dataSettings.columnsByBlock[EBlockKey.Series].slice(c_num+1))){
                        break;
                    }
                }
                let obj_array=c_lvl.children.filter(x=>x.name==row[col.path]);
                if(obj_array.length==0){
                    c_lvl.children.push({
                        name:row[col.path],
                        children:[]
                    })
                    obj_array=[c_lvl.children[c_lvl.children.length-1]];
                }
                c_lvl=obj_array[0]
            }
            c_lvl["value"]=row[value_col];
        }
        return ret;
    }
    private format_num(num:number){
        let opt={};
        if(this.viewSettings[EViewOption.EnableDigitCapacity]){
            opt={
                maximumFractionDigits:parseInt(this.viewSettings[EViewOption.DigitCapacity]),
                minimumFractionDigits:parseInt(this.viewSettings[EViewOption.DigitCapacity])
            }
        }
        for(let item of yDigitsGroup){
            const [value, ru, en,mul] = item;
            if(this.viewSettings[EViewOption.NumberGroups]==value)
                return (new Intl.NumberFormat('ru-Ru',opt)).format(num/mul)+((value=='')?(value):(' '+value))
        }
    }
    private simpleFormat(num:number){
        return (new Intl.NumberFormat('ru-Ru')).format(num)
    }

    getColors(){
        let val_to_color:any={};
        let cur_color=-1;
        let vals=this.data.map(x=>x[this.dataSettings.columnsByBlock[EBlockKey.Series][0].path])
        let ret=[];
        for(let i=0;i<vals.length;i++){
            if(val_to_color[vals[i]]==undefined){
               cur_color=(cur_color+1)%this.theme.colors.length; 
               val_to_color[vals[i]]=this.theme.colors[cur_color];
            }
            ret.push(val_to_color[vals[i]]);
        }
        return ret;
    }
    private getCategoryVal(category:any,col:any){
        let val_col=this.dataSettings.columnsByBlock[EBlockKey.Value][0].path;
        let val=0;
        this.data.forEach(x=>{
            if(x[col.path]==category){
                val+=x[val_col];
            }
        });
        return val;
    }
    private trace1:any;
    private resizeTimeout:any=undefined;
    private doPlot(force:boolean=true){
        if(force){
        var data = [ this.trace1 ];
          
            
        var layout = {    showlegend:true,  yaxis: {
      //    automargin: true
        },
        xaxis: {
        //  automargin: true
        },
        margin:{
            //l:10,
           l:    this.leftMargin,
            r:   this.rightMargin,
            t:20,
            b:20
        },
          autosize:true,    plot_bgcolor:"transparent",paper_bgcolor:'transparent'};
          document.querySelector("#root")!.innerHTML="";
        this.myPlot = (window as any).Plotly.newPlot('root', data, layout,
        {displayModeBar: false});

        window.removeEventListener('resize', this.chartResize);
         window.addEventListener('resize', this.chartResize);
        }else{
            if(this.resizeTimeout){
                clearTimeout(this.resizeTimeout);
            }
            this.resizeTimeout=setTimeout(()=>{this.doPlot(true)},200)
        }
    }
    private chartResize = () => {
        console.log('resize');
        this.doPlot();
    };

    leftMargin:number=0;
    rightMargin:number=0;

    private renderWidget(): void {
        // if(this.viewSettings[EViewOption.ShowFact]){
        //     document.body.classList.remove("hide-fact");
        // }else{
        //     document.body.classList.add("hide-fact");
        // }
        if(this.viewSettings[EViewOption.ShowValue]){
            document.body.classList.remove("hide-value");
        }else{
            document.body.classList.add("hide-value");
        }
        this.trace1 = {
            type: 'parcats',
            dimensions: this.dataSettings.columnsByBlock[EBlockKey.Series].map((col)=>{
                return {
                    label:(this.viewSettings[EViewOption.ShowFactName])?col.name:undefined,
                    values:this.data.map(x=>x[col.path]),
                    categoryarray:[... (new Set(this.data.map(x=>x[col.path])))], 
                    categoryorder:'array',
                    ticktext:[... (new Set(this.data.map(x=>x[col.path])))].map(x=>{
                        let lbl_data=[];
                            lbl_data.push(x);
                            lbl_data.push(this.format_num(this.getCategoryVal(x,col)));
                        return lbl_data.join("<br>");
                    }),
                }
            }),
            counts: this.data.map(x=>x[this.dataSettings.columnsByBlock[EBlockKey.Value][0].path]),
            
            line:{
                color:this.getColors(),
                   shape: 'hspline',
                   hovertemplate:(this.viewSettings[EViewOption.Tooltip])?'%{count}':undefined
                
                },
             marker: {
                  color: '#C8A2C8',
                  line: {
                      width: 0
                  }
              },
              hoveron:'color',
              hoverinfo:(this.viewSettings[EViewOption.Tooltip])?'count':'none',
              hovertemplate:(this.viewSettings[EViewOption.Tooltip])?'%{category} %{xother}':undefined,
             legendwidth : "600",
             legendgrouptitle :{
                text:"eee"
             }, showlegend: true
          };
          //@ts-ignore
          //this.myPlot.then((x:any)=>{x.on('plotly_hover', function(data:any){console.log(data,(this as any))})})
          this.leftMargin=this.getMarginForFitText(this.trace1.dimensions[0].ticktext);
          this.rightMargin=this.getMarginForFitText(this.trace1.dimensions[this.trace1.dimensions.length-1].ticktext);
          console.log(this.leftMargin,this.rightMargin,this.trace1)
          this.doPlot(true);

    }
    myPlot:any;

    onLangChange(): void {
        this.onChange();
    }

    onThemeChange(): void {}

    onChange(): void {
        this.renderWidget();
        this.ready();
    }

    onInit(): void {
       // 
    }

    getMarginForFitText(val:string[]){
        let length=0;
        let maxlength=this.viewSettings[EViewOption.LengthText];
        for(let s of val){
            if(s.length>maxlength){
                s=s.slice(0,maxlength);
            }
            let w=getTextWidth(s,`"Open Sans", verdana, arial, sans-serif`)
            if(w>length)
                length=w;
        }
        return length;
      }
}


export function getTextWidth(text:string, font:string):number {
    // re-use canvas object for better performance
    const canvas = (getTextWidth as any).canvas || ((getTextWidth as any).canvas = document.createElement("canvas"));
    const context = canvas.getContext("2d");
    context.font = font;
    const metrics = context.measureText(text);
    return metrics.width;
  }
  